import os
from enum import Enum

from typing import Dict, List, Union
from pronouns_test_data.models import GeneratedTestData, PronounTestCase, ValidCharacters
from pronouns_test_data.generate import generate_test_data

__all__ = [
    'DataFormat',
    'load_published_data',
    'GeneratedTestData',
    'generate_test_data',
    'PronounTestCase',
    'ValidCharacters'
]


class DataFormat(Enum):
    model = 'model'
    dict_list = 'list'
    raw_json = 'raw_json'
    raw_csv = 'raw_csv'


def load_published_data(
        version: str = 'latest',
        data_format: DataFormat = DataFormat.model
) -> Union[GeneratedTestData, List[Dict[str, str]], str]:
    """

    :param version: The version to load (e.g., 1.0.0; defaults to 'latest')
    :param data_format: How you would like the data returned:
        DataFormat.model: Returns an instance of models.GeneratedTestData
        DataFormat.list: Returns [{"uwnetid": "foo", "pronoun": "bar", "use_case": "baz"}, ...]
        DataFormat.raw_json: Returns the raw json document, for you to parse.
        DataFormat.raw_csv: Returns the raw CSV document, for you to parse.
    :return: The version of the data requested in the format requested.
    """
    here_ = os.path.abspath(os.path.dirname(__file__))
    data_dir = os.path.join(here_, 'data')
    file_suffix = 'csv' if data_format == DataFormat.raw_csv else 'json'
    file_name = os.path.join(data_dir, f'{version}.{file_suffix}')
    with open(file_name) as f:
        raw_data = f.read()
    if data_format.value.startswith('raw'):
        return raw_data
    model = GeneratedTestData.parse_raw(raw_data)
    if data_format == DataFormat.dict_list:
        return model.dict()['test_cases']
    return model



import string
from typing import List

from pydantic import BaseModel, Field
from inflection import camelize


class _BaseModel(BaseModel):
    class Config:
        @staticmethod
        def camelize_(value: str) -> str:
            return camelize(value, uppercase_first_letter=False)

        allow_population_by_field_name = True
        alias_generator = camelize_


class ValidCharacters(str):
    @staticmethod
    def illegal_punctuation() -> str:
        return "|\\"

    @classmethod
    def valid_punctuation(cls) -> str:
        punctuation = string.punctuation
        for char in cls.illegal_punctuation():
            punctuation = punctuation.replace(char, "")
        return punctuation

    @classmethod
    def valid_ascii(cls):
        return string.ascii_letters + string.digits + cls.valid_punctuation() + " "


class PronounTestCase(_BaseModel):
    uwnetid: str
    pronoun: str
    use_case: str = Field(
        ..., description="A description of what this case is testing."
    )


class GeneratedTestData(BaseModel):
    test_cases: List[PronounTestCase] = []



from enum import Enum
from typing import NoReturn, cast

import requests
import json
import sys
import os

import typer

from pronouns_test_data.models import GeneratedTestData, PronounTestCase

CERT_FILE = "/usr/local/lib/subman/certs/wmango.cert"
KEY_FILE = "/usr/local/lib/subman/certs/wmango.key"


class IRWSEndpoint(Enum):
    eval = "eval"
    prod = "prod"

    def get_url_for_netid(self, netid: str) -> str:
        return f"{self.url}/v3/entity.profile/uwnetid={netid}?-reflect"

    @property
    def url(self):
        return _ENDPOINTS[self]


_ENDPOINTS = {
    IRWSEndpoint.eval: "https://mango-eval.u.washington.edu:646/registry-eval",
    IRWSEndpoint.prod: "https://mango.u.washington.edu:646/registry",
}


class PronounsClient:
    def __init__(
        self,
        endpoint: IRWSEndpoint,
        cert_path: str,
        key_path: str,
        dry_run: bool = True,
    ):
        self.endpoint = endpoint
        self.cert_path = cert_path
        self.key_path = key_path
        self.dry_run = dry_run

    @staticmethod
    def get_payload(pronoun: str) -> str:
        """Gets the valid IRWS put payload for the given pronoun."""
        return json.dumps({"entity.profile": [{"pronoun": pronoun}]})

    def _publish_update(self, url: str, payload: str) -> NoReturn:
        if self.dry_run:
            typer.echo(f"{url} => {payload}")
            return

        response = requests.put(
            url,
            data=payload,
            cert=(self.cert_path, self.key_path),
            headers={"Content-type": "application/json", "Accept": "application/json"},
        )
        response.raise_for_status()

    def update_test_case(self, case: PronounTestCase) -> NoReturn:
        url = self.endpoint.get_url_for_netid(case.uwnetid)
        payload = self.get_payload(case.pronoun)
        self._publish_update(url, payload)


def app(
    input_file: str = typer.Option(
        ..., "--in", "-i", help="The JSON file containing the test data."
    ),
    cert_file: str = typer.Option(
        ...,
        "--cert",
        "-c",
        help="The path to the cert file you want to use for the connection.",
    ),
    key_file: str = typer.Option(
        ...,
        "--key",
        "-k",
        help="The path to the key file that goes with the cert file.",
    ),
    target_endpoint: IRWSEndpoint = typer.Option(
        IRWSEndpoint.eval.value, "--target", "-t", help="The IRWS api endpoint to use."
    ),
    dry_run: bool = typer.Option(
        True,
        "--publish",
        "-p",
        help="This runs in dry-run mode by default. Use this option to actually push data.",
    ),
):
    abort = False
    if not os.path.exists(input_file):
        typer.echo(f"Input file '{input_file}' does not exist!", color=typer.colors.RED)
        abort = not dry_run
    if not os.path.exists(cert_file):
        typer.echo(
            f"SSL cert file '{cert_file}' does not exist", color=typer.colors.RED
        )
        abort = not dry_run
    if not os.path.exists(key_file):
        typer.echo(f"SSL key file '{key_file}' does not exist", color=typer.colors.RED)
        abort = not dry_run

    if abort:
        typer.Exit()

    client = PronounsClient(target_endpoint, cert_file, key_file, dry_run)
    test_data = GeneratedTestData.parse_file(input_file)
    for profile in test_data.test_cases:
        typer.echo(
            f"Setting pronoun for UW NetID '{profile.uwnetid}' to {profile.pronoun}"
        )
        try:
            client.update_test_case(profile)
        except requests.HTTPError as e:
            response = cast(requests.Response, e.response)
            message = (
                f'FAILED: [{response.status_code}] {response.content.decode("UTF-8")}'
            )
            typer.echo(message, color=typer.colors.RED)


if __name__ == "__main__":
    typer.run(app)



from __future__ import annotations

import csv
import os
import random
import string
from enum import Enum
from typing import NoReturn

import typer

from pronouns_test_data.models import (
    GeneratedTestData,
    PronounTestCase,
    ValidCharacters,
)

__all__ = [
    'generate_test_data'
]


def generate_jargon(valid_chars: str, length: int) -> str:
    return "".join([random.choice(valid_chars) for _ in range(length)])


def write_raw_data(filename: str, data: str) -> NoReturn:
    with open(filename, 'w') as f:
        f.write(data)
    typer.echo(f"Wrote JSON data to {filename}")


def write_csv_data(filename: str, data: GeneratedTestData):
    with open(filename, 'w', newline='') as f:
        writer = csv.DictWriter(f, fieldnames=list(PronounTestCase.__fields__.keys()))
        writer.writeheader()
        for case in data.test_cases:
            writer.writerow(case.dict())
    typer.echo(f"Wrote CSV data to {filename}")


def generate_test_data() -> GeneratedTestData:
    return GeneratedTestData(
        test_cases=[
            PronounTestCase(
                uwnetid="javerage",
                pronoun="they/them/their/theirs/themself",
                use_case="The full expression of a single pronoun set. "
                "Also, this netid should be registered for some university classes.",
            ),
            PronounTestCase(
                uwnetid="uwitpn01",
                pronoun="she/her/hers; they/them/theirs",
                use_case="A consistently formatted list of two pronoun sets.",
            ),
            PronounTestCase(
                uwnetid="uwitpn02",
                pronoun="he him his",
                use_case="A single, atypically formatted pronoun set.",
            ),
            PronounTestCase(
                uwnetid="uwitpn03",
                pronoun="she, her, hers or they/them",
                use_case="Two atypically and inconsistently formatted pronoun sets.",
            ),
            PronounTestCase(
                uwnetid="uwitpn04",
                pronoun="please use my name",
                use_case="Guidance in place of a declared pronoun set, with no punctuation.",
            ),
            PronounTestCase(
                uwnetid="uwitpn05",
                pronoun="i am exploring this. please ask me!",
                use_case="Guidance in place of a declared pronoun set, with punctuation.",
            ),
            PronounTestCase(
                uwnetid="uwitpn06",
                pronoun="they/them/theirs;they/them/theirs;they/them/theirs",
                use_case="A consistently formatted pronoun set that is (accidentally?) pasted three times, with no spaces.",
            ),
            PronounTestCase(
                uwnetid="uwitpn07",
                pronoun="she/they",
                use_case="A common shorthand expression of two pronoun sets.",
            ),
            PronounTestCase(
                uwnetid="uwitpn08",
                pronoun="ze/hir",
                use_case="A common shorthand expression of two neo-pronoun sets.",
            ),
            PronounTestCase(
                uwnetid="uwitpn09",
                pronoun="fae/r or they/themself",
                use_case='One neo-pronoun shorthand and one shorthand expression of the "they" pronoun set, each with its '
                "own shorthand syntax",
            ),
            PronounTestCase(
                uwnetid="uwitpn10",
                pronoun='""el"&maybe "els" or just ask?',
                use_case="A pronoun, plus guidance, with inconsistent punctuation.",
            ),
            PronounTestCase(
                uwnetid="uwitpn11",
                pronoun=generate_jargon(ValidCharacters.valid_ascii(), 12),
                use_case="12 randomly generated characters (very common length)",
            ),
            PronounTestCase(
                uwnetid="uwitpn12",
                pronoun=generate_jargon(ValidCharacters.valid_ascii(), 16),
                use_case="16 randomly generated characters (very common length)",
            ),
            PronounTestCase(
                uwnetid="uwitpn13",
                pronoun=generate_jargon(ValidCharacters.valid_ascii(), 32),
                use_case="32 randomly generated characters (uncommon length)",
            ),
            PronounTestCase(
                uwnetid="uwitpn14",
                pronoun=generate_jargon(ValidCharacters.valid_ascii(), 64),
                use_case="64 randomly generated characters (unlikely length)",
            ),
            PronounTestCase(
                uwnetid="uwitpn15",
                pronoun=generate_jargon(ValidCharacters.valid_ascii(), 128),
                use_case="128 randomly generated characters (very unlikely length)",
            ),
            PronounTestCase(
                uwnetid="uwitpn16",
                pronoun=generate_jargon(ValidCharacters.valid_ascii(), 140),
                use_case="140 (max. length) randomly generated characters (very unlikely length)",
            ),
            PronounTestCase(
                uwnetid="uwitpn17",
                pronoun=generate_jargon(ValidCharacters.valid_punctuation(), 140),
                use_case="140 characters of only punctuation, to test text wrapping",
            ),
            PronounTestCase(
                uwnetid="uwitpn18",
                pronoun=generate_jargon(string.digits, 140),
                use_case="140 characters of only punctuation, to test text wrapping",
            ),
        ]
    )


class OutputType(Enum):
    json = 'json'
    csv = 'csv'


def app(
    out: str = typer.Option(
        "data",
        "--output-path",
        "-o",
        help="The directory "
    ),
    file_prefix: str = typer.Option(
        "iam-pronouns-test-data",
        "--file-prefix",
        "-f",
        help="The filename to use before the filetype extension (e.g., the `foo` in `foo.json`)"
    )
):
    test_data = generate_test_data()
    if not os.path.exists(out):
        os.makedirs(out, exist_ok=True)

    json_out = os.path.join(out, f'{file_prefix}.json')
    csv_out = os.path.join(out, f'{file_prefix}.csv')

    test_json = test_data.json(by_alias=True, indent=4)
    write_raw_data(json_out, test_json)
    write_csv_data(csv_out, test_data)


if __name__ == "__main__":
    typer.run(app)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pronouns_test_data']

package_data = \
{'': ['*'], 'pronouns_test_data': ['data/*']}

install_requires = \
['inflection>=0.5.1,<0.6.0',
 'pydantic>=1.7.3,<2.0.0',
 'requests>=2.25.1,<3.0.0',
 'typer>=0.3.2,<0.4.0']

setup_kwargs = {
    'name': 'pronouns-test-data',
    'version': '1.0.0',
    'description': 'A library to aid in vending test data for UW-IT IAM pronouns test cases.',
    'long_description': None,
    'author': 'Tom Thorogood',
    'author_email': 'tom@tomthorogood.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
